package lcs;

/**
 * A LCSProblem represents a longest common subsequence problem for strings.
 * A LCS problem is specified by two strings.
 * It can be used as a key in a solutions table.
 */
public class LCSProblem {

  private String string1;
  private String string2;

  /**
   * new LCSProblem(string1, string2) returns a new LCS problem for the longest
   * common subsequence between string1 and string2.
   * @param string1
   * @param string2
   */
  public LCSProblem(String string1, String string2) {
    this.string1 = string1;
    this.string2 = string2;
  }

  /**
   * prob.getString1() returns the first string for prob.
   * @return the first string
   */
  public String getString1() {
    return string1;
  }

  /**
   * prob.getString2() returns the second string for prob.
   * @return the second string
   */
  public String getString2() {
    return string2;
  }

  /**
   * prob.equals(obj) returns true if obj is a LCS problem with the same strings
   * as prob.
   * @param obj
   * @return true if the problems have the same strings
   */
  @Override
  public boolean equals(Object obj) {
    if (!(obj instanceof LCSProblem)) {
      return false;
    }
    LCSProblem other = (LCSProblem) obj;
    return string1.equals(other.string1) && string2.equals(other.string2);
  }

  /**
   * prob.hashCode() returns a hash code for prob.
   * If two LCS problems are equal then they have the same hash code.
   * @return
   */
  @Override
  public int hashCode() {
    int hashCode1 = string1.hashCode();
    int hashCode2 = string2.hashCode();
    return hashCode1 ^ (hashCode2 << 16 | hashCode2 >>> 16);
  }

  /**
   * prob.toString() returns a description for prob.
   * This string consists of the two strings for prob separated by a comma.
   * @return
   */
  @Override
  public String toString() {
    return string1 + ", " + string2;
  }

}
