package totalprice;

import gui.Log;
import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JSpinner;
import javax.swing.SpinnerNumberModel;

/**
 * The Tester class provides a static main() method that brings up a graphical
 * user interface for testing the total price optimizer and total price solver
 * classes.
 */
public class Tester {

  private JPanel contentPane;
  private PriceList priceList;
  private SpinnerNumberModel lengthModel;

  private Action createAction(final TotalPriceSolver solver) {
    return new AbstractAction(solver.toString()) {

      @Override
      public void actionPerformed(ActionEvent ev) {
        Log.clear();
        Log.log(solver);
        Solution solution =
                solver.getBestSolution(lengthModel.getNumber().intValue());
        Log.log(solution);
      }

    };
  }

  private Action createAction(final TotalPriceOptimizer optimizer) {
    return new AbstractAction(optimizer.toString()) {

      @Override
      public void actionPerformed(ActionEvent ev) {
        Log.clear();
        Log.log(optimizer);
        int length = lengthModel.getNumber().intValue();
        double bestTotalPrice = optimizer.bestTotalPrice(length);
        Log.log("For length " + length + ", best total price = "
                + bestTotalPrice + ", call count = " + Log.getCallCount());
      }

    };
  }

  private void createContentPane() {
    contentPane = new JPanel(new BorderLayout());
    JPanel actionPanel = new JPanel(new GridLayout(6, 1));
    priceList = new PriceList(1.0, 10);
    lengthModel = new SpinnerNumberModel(10, 1, 20, 1);
    JPanel logControlPanel = new JPanel(new FlowLayout());
    logControlPanel.add(new JCheckBox(Log.getSetTraceCallsAction()));
    Log.setTraceCalls(false);
    logControlPanel.add(new JSpinner(lengthModel));
    logControlPanel.add(new JLabel("Bar Length"));
    actionPanel.add(logControlPanel);
    TotalPriceOptimizer optimizer1 =
            new SimpleTotalPriceOptimizer(priceList);
    actionPanel.add(new JButton(createAction(optimizer1)));
    TotalPriceOptimizer optimizer2 =
            new OrderedTotalPriceOptimizer(priceList);
    actionPanel.add(new JButton(createAction(optimizer2)));
    TotalPriceOptimizer optimizer3 =
            new MemoizedSimpleTotalPriceOptimizer(priceList);
    actionPanel.add(new JButton(createAction(optimizer3)));
    TotalPriceOptimizer optimizer4 =
            new MemoizedOrderedTotalPriceOptimizer(priceList);
    actionPanel.add(new JButton(createAction(optimizer4)));
    TotalPriceSolver solver = new MemoizedSimpleTotalPriceSolver(priceList);
    actionPanel.add(new JButton(createAction(solver)));
    JPanel outerPanel = new JPanel(new FlowLayout());
    outerPanel.add(actionPanel);
    contentPane.add(outerPanel, BorderLayout.WEST);
    JComponent logViewer = Log.createViewComponent();
    logViewer.setPreferredSize(new Dimension(700, 400));
    contentPane.add(logViewer, BorderLayout.CENTER);
  }

  public JComponent getContentPane() {
    if (contentPane == null) {
      createContentPane();
    }
    return contentPane;
  }

  public JMenuBar getMenuBar() {
    JMenuBar menuBar = new JMenuBar();
    JMenu logMenu = new JMenu("Log");
    logMenu.add(new JMenuItem(Log.getSaveAction()));
    menuBar.add(logMenu);
    return menuBar;
  }

  public static void main(String[] args) {
    Tester tester = new Tester();
    JFrame frame = new JFrame();
    frame.setContentPane(tester.getContentPane());
    frame.setJMenuBar(tester.getMenuBar());
    frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    frame.setLocation(100, 100);
    frame.pack();
    frame.setVisible(true);
  }

}
